######### global variables #########

RDHOME="$HOME/Rdaemon"

######### functions #########

#check whether Rdaemon runs
runs() {
	RUBYRUN=$(ps xw | grep '[0-9] ruby.*Rdaemon')
	RRUN=$(ps aw | grep '[0-9] /Lib.*TMRdaemon')
	if [ ${#RRUN} == 0 -a ${#RUBYRUN} == 0 ]; then
		return 0
	else
		return 1
	fi
}

######### begin script #########

#check whether R is already running
runs
if [ $? == 0 ]; then
	echo -en "Rdaemon is not running!"
	exit 206
fi

#get the task from TM and delete beginning >+:
IN=$1
TASK=$(echo "$IN" | sed -e 's/^[>+:]//;s/^[ ]*//')
LENTASK=${#IN}
LENTASK=$((LENTASK+2))
# @| indicates that this command will not be inserted into the history
[[ "${IN:0:2}" == "@|" ]] && LENTASK=$((LENTASK-2))

#check whether the task is empty
if [ "${#TASK}" == "0" -o "$TASK" == " " ]; then
	echo -en "\n> "
	exit 203
fi

#check named input pipe for safety reasons
if [ ! -p "$RDHOME"/r_in ]; then
	echo -en "Rdaemon Error:\nThe pipe /tmp/r_in is not found!\n\nYou have to kill Rdaemon manually!"
	exit 206
fi

#delete default pdf printout
#[[ -e "$RDHOME"/plots/Rplot.pdf ]] && rm "$RDHOME"/plots/Rplot.pdf

#set history counter to 0
echo -n 0 > "$HOME/Rdaemon/history"/Rhistcounter.txt

#get current position of r_out and r_err
POS=$(stat "$RDHOME"/r_out | awk '{ print $8 }')
POSERR=$(stat "$RDHOME"/r_err | awk '{ print $8 }')

#send task to Rdaemon
echo -e "$TASK" > "$RDHOME"/r_in

#wait for the Rdaemon - help wanted!!!!!
S=0
RPID=$(ps aw | grep '[0-9] /Lib.*TMRdaemon' | awk '{print $1;}' )
if [ "${#RPID}" == 0 ]; then
	echo -en "Rdaemon was stopped."
	exit 206
fi
#loop unless cpu coverage of R is zero
sleep 0.02 # for safety reasons
while [ "$S" == "0" ]
do
	cpu=$(ps -eo pcpu,pid | grep "\b$RPID\b" | awk '{print $1;}')
	[[ -z "$cpu" ]] && exit 200
	[[ "$cpu" < "1.0" ]] && break
	CP=$(echo -n "$cpu" | perl -e 'print 100-<>')
	echo "$CP Please wait. Rdaemon is busy ..."
	sleep 0.4
done|CocoaDialog progressbar --title "Rdaemon"
sync
#read only the current response from Rdaemon
POSNEW=$(stat "$RDHOME"/r_out | awk '{ print $8 }')
OFF=$(($POSNEW - $POS + 2))
RES=$(tail -c $OFF "$RDHOME"/r_out)

#check for errors
# 1. user errors
POSERRNEW=$(stat "$RDHOME"/r_err | awk '{ print $8 }')
if [ "$POSERR" != "$POSERRNEW" ]; then
	OFF=$(($POSERRNEW - $POSERR + 1))
	RES=$(tail -c $OFF "$RDHOME"/r_err)
	echo "$RES"
	exit 206
else
# 2. internal errors - wait 8 seconds for R
	if [ "> $TASK" == "$RES" ]; then
		sync
		touch -f "$RDHOME"/r_err
		#timeout for internal errors
		sleep 8 | (CocoaDialog progressbar  --indeterminate --title Rdaemon --text 'Try to retrieve data! Please wait ...')
		sync
		touch -f "$RDHOME"/r_err
		POSERRNEW=$(stat "$RDHOME"/r_err | awk '{ print $8 }')
		# error found ?
		if [ "$POSERR" != "$POSERRNEW" ]; then
			OFF=$(($POSERRNEW - $POSERR + 1))
			RES=$(tail -c $OFF "$RDHOME"/r_err)
			echo "$RES"
			exit 206
		else
			# output found ?
			POSNEW=$(stat "$RDHOME"/r_out | awk '{ print $8 }')
			OFF=$(($POSNEW - $POS + 2))
			RES=$(tail -c $OFF "$RDHOME"/r_out)
			# should be an error
			if [ -z "$RES" ]; then 
				echo -e "Probaly an error occured. Please check manually!";
				exit 206
			else
				if [ ! -z "$TM_SELECTED_TEXT" ]; then
					echo -e "$RES" > "$RDHOME"/r_res
					echo "Result is written to 'r_res'"
					exit 206
				fi
				# I DO NOT KNOW WHY?? For long input lines \x0d and \x08 occurs in the terminal output.
				RES=$(echo "$RES" | perl -e '
					undef($/); $a=<>;
					$a=~s/\x0D{1,}/\x0D/sg;
					while($a=~m/\x0D|\x08/) { $a=~s/(.*?)\x0D<.{50}(.) +\x08+(.*)/$1$2$3/sg; }
					print "$a";
				')
				echo -en "${RES:$LENTASK}"
			fi
		fi
	else
# no error
		if [ ! -z "$TM_SELECTED_TEXT" ]; then
			echo -e "$RES" > "$RDHOME"/r_res
			echo "Result is written into 'r_res'"
			exit 206
		fi
		# I DO NOT KNOW WHY?? For long input lines \x0d and \x08 occurs in the terminal output.
		RES=$(echo "$RES" | perl -e '
			undef($/); $a=<>;
			$a=~s/\x0D{1,}/\x0D/sg;
			while($a=~m/\x0D|\x08/) { $a=~s/(.*?)\x0D<.{50}(.) +\x08+(.*)/$1$2$3/sg; }
			print "$a";
		')
		echo -en "${RES:$LENTASK}"
	fi
fi
