###########
#
# Rdaemon start script
#
#   Hans-Jörg Bibiko - bibiko@eva.mpg.de
#
###########

######### global variables #########

RDHOME="$HOME/Rdaemon"

######### functions #########

runs() {
	RUBYRUN=$(ps xw | grep '[0-9] ruby.*Rdaemon')
	RRUN=$(ps aw | grep '[0-9] /Lib.*TMRdaemon')
	if [ ${#RRUN} == 0 -a ${#RUBYRUN} == 0 ]; then
		return 0
	else
		return 1
	fi
}

######### script begin #########

#check whether R is already runing
runs
if [ $? == 1 ]; then
	echo -en "Rdaemon is already running!"
	exit 206
fi

#start X11
export DISPLAY=:0.0
open -a X11
osascript <<-AS &>/dev/null
 	tell application "TextMate"
 		activate
 	end tell
AS

#create RDHOME
#[[ ! -d "$RDHOME" ]] && mkdir "$RDHOME"

#create plot directory
[[ ! -d "$RDHOME/plots" ]] && mkdir "$RDHOME/plots"

#create history directory
[[ ! -d "$RDHOME/history" ]] && mkdir "$RDHOME/history"

#create the named pipe /tmp/r_in
if [ ! -e "$RDHOME"/r_in ]; then
	mkfifo "$RDHOME"/r_in
else
	if [ ! -p "$RDHOME"/r_in ]; then
		rm "$RDHOME"/r_in
		mkfifo "$RDHOME"/r_in
	fi
fi
sleep 0.5

#delete r_out 
[[ -f "$RDHOME"/r_out ]] && rm "$RDHOME"/r_out

#copy console.r not exists 
if [ ! -f "$RDHOME"/console.r ]; then
	cp "$TM_BUNDLE_SUPPORT"/bin/console.r "$RDHOME"/console.r
fi
#copy Rsession TMProject if not exists 
if [ ! -f "$RDHOME"/Rsession.tmproj ]; then
	cp "$TM_BUNDLE_SUPPORT"/bin/Rsession.tmproj "$RDHOME"/Rsession.tmproj
fi
#copy startOptions.R if not exists 
if [ ! -f "$RDHOME"/startOptions.R ]; then
	cp "$TM_BUNDLE_SUPPORT"/bin/startOptions.R "$RDHOME"/startOptions.R
fi
#copy Rhistory.txt if not exists 
if [ ! -f "$RDHOME"/history/Rhistory.txt ]; then
	cp "$TM_BUNDLE_SUPPORT"/bin/Rhistory.txt "$RDHOME"/history/Rhistory.txt
fi

#remove duplicated lines of /Rhistory.txt
cat "$RDHOME"/history/Rhistory.txt | uniq > "$RDHOME"/history/Rhistory.temp
rm "$RDHOME"/history/Rhistory.txt
cat "$RDHOME"/history/Rhistory.temp > "$RDHOME"/history/Rhistory.txt
rm "$RDHOME"/history/Rhistory.temp

#start Rdaemon
"$RDHOME"/daemon/startR.sh &> /dev/null &

#reset history counter
echo -n 0 > "$HOME/Rdaemon"/history/Rhistcounter.txt

#safety counter
SAFECNT=0
#wait until r_out is created (or not) by the Rdaemon
while [ ! -f "$RDHOME"/r_out ]
do
	SAFECNT=$(($SAFECNT+1))
	if [ $SAFECNT -gt 50000 ]; then
		echo -en "Start failed! No response from Rdaemon!"
		exit 206
	fi
done

#wait for Rdaemon's output is ready
S=0
CNT=1
while [ "$S" == 0 ]
do
	ST=$(tail -n 1 "$RDHOME"/r_out )
	if [ "$ST" == "> " ]; then
		S=1
	fi
	CNT=$(($CNT+15))
	echo "$CNT Rdaemon is starting ..."; sleep .5
done|CocoaDialog progressbar --title "Rdaemon"

runs
if [ $? == 1 ]; then
	echo -en "> "
else
	echo "While starting Rdaemon was shutdown unexpectedly!"
	exit 206
fi
